#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdbool.h>
#include <unistd.h>
#include <stdio.h>
#include <jni.h>
#include <signal.h>
#include <pthread.h>
#include <stdlib.h>

#ifndef _LIBS_CUTILS_LOG_H
#include "logdebug.h"
#endif
//Javaで定義したネイティブメソッドのJNIヘッダをincludeする
#include "my_pfsample_barcode_RxtxSerial.h"

#define FALSE 0
#define TRUE 1

struct termios oldtio, newtio; /* 通信設定構造体 */
int BAUDRATE = B9600; /* 通信速度 */
static int fd = 0; /* デバイスファイルディスクリプタ */
static bool debug = FALSE; /* デバッグフラグ */
static bool STOP = FALSE;
int res = 0;

void *readThread(void *arg); /* シリアル入力データ受信用のスレッド */
int wait_flag = TRUE; /* シグナルを受け取っていなければ TRUE */

JavaVM *gJavaVM;
jmethodID mid;
jclass mClass;
jobject mObject;
bool isAttached = false;

jint JNI_OnLoad(JavaVM* vm, void* reserved) {

	JNIEnv *env;
	gJavaVM = vm;
	int result;
	LOGD_IF(debug, "****************JNI: +JNI_OnLoad");
	result = JNI_VERSION_1_6;
	LOGD_IF(debug, "*******************JNI: -JNI_OnLoad");
	return result;
}

/********************************************************
 * シリアルポートのオープン(デバッグコントロール付)
 * @param dev : デバイスファイル名( ex : "/dev/ttyS0" )
 * @param debug : true/デバッグOn false/デバッグOff
 * @return 成功 > 0
 * @return 失敗 <= 0
 **********************************************************/

JNIEXPORT jint JNICALL Java_jp_oesf_ensyu_RxtxSerial_openComm(JNIEnv *env,
		jobject obj, jstring dev, jboolean debug_flag) {
	// デバッグフラグの設定
	debug = debug_flag;
	LOGD_IF(debug, "JNI: +openPort");
	jboolean isCopy = JNI_FALSE;
	const char *port = (*env)->GetStringUTFChars(env, dev, &isCopy);

	LOGD_IF( debug, "JNI: openPort: Device Open...: %s", port);

	// シリアルのオープン(読み取り専用)
	fd = open(port, O_RDONLY);
	if (fd < 0) {
		LOGD_IF(debug, "JNI: openPort: open Fail!\n");
		return FALSE;
	}
	if (JNI_TRUE == isCopy) {
		(*env)->ReleaseStringUTFChars(env, dev, port);
	}
	LOGD_IF(debug, "JNI: -openPort");
	return TRUE;
}

/*****************************************
 * 通信パラメータの設定
 *
 * @param baudRate : ボーレート
 * @param dataBits : データビット長
 * @param stopBits : ストップビット長
 * @param parity   : パリティ設定
 * @return true :　成功
 * @return false : 失敗
 *******************************************/
JNIEXPORT jboolean JNICALL Java_jp_oesf_ensyu_RxtxSerial_setCommParams(
		JNIEnv *env, jobject obj, jint baudrate, jint databits, jint stopbits,
		jint parity) {

	debug = TRUE;
	LOGD_IF(debug, "JNI: +setParam");
	/* 現在の設定をデバッグ表示 */
	LOGD_IF(debug, "Now Setteing");
	LOGD_IF(debug, "Input mode %x", oldtio.c_iflag);
	LOGD_IF(debug, "output mode %x", oldtio.c_oflag);
	LOGD_IF(debug, "Control mode %x", oldtio.c_cflag);
	LOGD_IF(debug, "Local mode %x", oldtio.c_lflag);
	LOGD_IF(debug, "Baud Rate %d", cfgetospeed(&oldtio));

	BAUDRATE = translate_speed(env, baudrate);

	/* 現在の通信設定の退避 */
	tcgetattr(fd, &oldtio);

	/* 新しいポートの設定をカノニカル入力処理に設定する */
	//制御モード（シリアル通信の制御全般の設定）
	newtio.c_cflag = BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;
	//入力モード(入力された文字に対して行う設定)
	newtio.c_iflag = IGNPAR | ICRNL;
	//出力モード(出力する文字ｊに対して行う処理)
	newtio.c_oflag = 0;
	//ローカルモード（通信先に影響のない設定）
	newtio.c_lflag = ICANON;
	newtio.c_cc[VMIN] = 1;
	newtio.c_cc[VTIME] = 0;

	cfmakeraw(&newtio);
	cfsetospeed(&newtio, BAUDRATE);
	cfsetispeed(&newtio, BAUDRATE);
	/* 設定上書き(即座に) */
	tcflush(fd, TCIFLUSH);
	tcsetattr(fd, TCSANOW, &newtio);

	/* 設定値のデバッグ表示 */
	LOGD_IF(debug, "New Setteing");
	LOGD_IF(debug, "Input mode %x", newtio.c_iflag);
	LOGD_IF(debug, "output mode %x", newtio.c_oflag);
	LOGD_IF(debug, "Control mode %x", newtio.c_cflag);
	LOGD_IF(debug, "Local mode %x", newtio.c_lflag);
	LOGD_IF(debug, "Baud Rate %d", cfgetospeed(&newtio));
	LOGD_IF(debug, "JNI: -setParam");
	return JNI_TRUE;
}

/***********************************************
 * このインスタンスをネイティブに登録する
 * @param listener : 自インスタンス
 ************************************************/
JNIEXPORT void JNICALL Java_jp_oesf_ensyu_RxtxSerial_setListener(
		JNIEnv *env, jclass clazz, jobject obj) {

	LOGD_IF(debug, "JNI: +setListener");

	jclass claz = (*env)->GetObjectClass(env, obj);
	mClass = (jclass) (*env)->NewGlobalRef(env, claz);

	mObject = (*env)->NewGlobalRef(env, obj); //グローバル参照取得
	mid = (*env)->GetStaticMethodID(env, clazz, "callBackFromJNI", "(Ljava/lang/String;)V");

	// バーコードからのデータ受信待ちのスレッドを生成
	pthread_t thread_;
	int status = pthread_create(&thread_, NULL, readThread, NULL);
	if (status != 0) {
		LOGD_IF(debug, "JNI: setListener: createThread Fail!\n");
		return;
	}
	LOGD_IF(debug, "JNI: -setListener");
	return;
}

/**********************************************
 * 受信まちスレッド
 *********************************************/

void *readThread(void *arg) {
	LOGD_IF(debug, "in readThread !!\n");

	//JavaにコールバックするためにJavaVMのJNIEnvを取得してこのスレッドにアタッチさせる
	int status;
	JNIEnv *env;
	isAttached = false;
	usleep(100000);
	status = (*gJavaVM)->GetEnv(gJavaVM, (void **) &env, JNI_VERSION_1_6);
	if (status < 0) {
		LOGD_IF(debug,
				"callback_handler: failed to get JNI environment assuming native thread");
		status = (*gJavaVM)->AttachCurrentThread(gJavaVM, &env, NULL);
		if (status < 0) {
			LOGD_IF(debug, "callback_handler: failed to attach current thread");
			return NULL;
		}
		isAttached = true;
	}


	// 受信データ入力待ちの間ループ
		int size = 1024; //文字列サイズ
		char buf[size]; //バーコードの文字列バッファ
		char *pos = &buf[0];
		char c;

	while (!STOP) {
		res = read(fd, &c, 1);
		if(res == 0){
			continue;
		}else if(res < 0){
			LOGE_IF(debug, "エラー：不正なデータ受信\n");
			return NULL;
		}
		switch((unsigned char)c){
			case 0x0C:
				//データの始まり
				LOGD_IF(debug, "0Cを受信\n");
				break;
			case 0x0D:
				//データの終わり
				LOGD_IF(debug, "0Dを受信\n");
				*pos = '\0';
				LOGD_IF(debug, "%s\n", buf);
				//文字列をjstringにする
				jstring result = (*env)->NewStringUTF(env, buf);
				//Androidアプリの画面に読み取った文字列をコールバックする
				(*env)->CallStaticVoidMethod(env, mClass, mid, result);
				pos = buf;
				break;
			default:
				if((pos - buf) < size){
					*pos++ = c;
				}
				break;
		}
	}
	return NULL;
}

/****************************************
 * シリアルポートのクローズ
 *****************************************/

JNIEXPORT void JNICALL Java_jp_oesf_ensyu_RxtxSerial_closeComm(JNIEnv *env,
		jobject obj) {

	LOGD_IF(debug, "JNI: +closeComm");

	STOP = true;

	if (mObject != NULL) {
		(*env)->DeleteGlobalRef(env, mObject);
	}
	mObject = NULL;
	LOGD_IF(debug, "JNI: DeleteGlobalRef DONE");

	// 強制中断のためにSIGINTを送信
	raise(SIGINT);
	// 保管しておいた設定を元に戻す
	ioctl(fd, TCSETS, &oldtio);
	// シリアルデバイスのクローズ
	close(fd);

	if (isAttached) {
		(*gJavaVM)->DetachCurrentThread(gJavaVM);
		LOGD_IF(debug, "JNI: DetachCurrentThread DONE");
	}

	LOGD_IF(debug, "JNI: -closeComm");

	return;
}


int main(int argc, char *argv[]) {
	int i;

	for (i = 1; i < argc; ++i) /* コマンドライン引数を出力するループ */
	{
		puts(argv[i]);
	}

	return 0;
}

int translate_speed(JNIEnv *env, jint speed) {
	switch (speed) {
	case 0:
		return B0;
	case 50:
		return B50;
	case 75:
		return B75;
	case 110:
		return B110;
	case 134:
		return B134;
	case 150:
		return B150;
	case 200:
		return B200;
	case 300:
		return B300;
	case 600:
		return B600;
	case 1200:
		return B1200;
	case 1800:
		return B1800;
	case 2400:
		return B2400;
	case 4800:
		return B4800;
	case 9600:
		return B9600;
	case 19200:
		return B19200;
	case 38400:
		return B38400;
	case 57600:
		return B57600;
	case 115200:
		return B115200;
	}

	/* Handle custom speeds */
	if (speed >= 0)
		return speed;
	else {
		return -1;
	}
}
