package my.pfsample.barcode;

import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;


public class MainActivity extends Activity implements OnSerialListener {
	private final static String TAG = "MainActivity";

	private RxtxSerial sp;
	private static final int BAUDRATE = 9600;//通信速度9600bps
	private static final int DATABITS_8 = 8;//8bitデータ
	private static final int STOPBITS_1 = 1;//ストップビットは1
	private static final int PARITY_NONE = 0;//パリティなし
	private static final String SERIAL_PORT = "/dev/ttyUSB0";//デバイスファイル名

	private static final String[] appBook = {
			"Androidアプリケーション技術社認定試験ベーシック対策実践問題集", "OESF", "2800円", "日経BP" };
	private static final String noData = "取得できませんでした";
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		//RxtxSerialクラスのインスタンス生成
		sp = new RxtxSerial();

		// シリアルポートをオープンするネイティブメソッド呼び出し
		int devStatus = sp.openComm(SERIAL_PORT, true);

		// シリアルポートのオープンに成功したら、シリアル通信パラメータを設定する
		if (devStatus > 0) {
			sp.setCommMode(BAUDRATE, DATABITS_8, STOPBITS_1, PARITY_NONE);
		} else {
			Log.e(TAG, "Error can't device open!");
			closeSpComm();
			return;
		}
		// OnSerialListenerを登録して読み取り開始
		sp.setOnSerialListener(this);
	}

	
	/**
	 * OnSerialListenerのイベントメソッド
	 */
	@Override
	public void onReceive(String id) {	
		// ネイティブ側でバーコードリーダーからデータを受信したとき
		// RxtxSerialを介して呼ばれる

		Toast.makeText(getApplicationContext(), "id = " + id,
				Toast.LENGTH_SHORT).show();

		TextView jan = (TextView) findViewById(R.id.jan);
		jan.setText(id);

		String isbn = id.substring(3, id.length() - 1);

		TextView isbnView = (TextView) findViewById(R.id.isbn);
		isbnView.setText(isbn);
		
		TextView titleView = (TextView) findViewById(R.id.title);
		TextView autherView = (TextView) findViewById(R.id.auther);
		TextView valueView = (TextView) findViewById(R.id.value);
		TextView publishView = (TextView) findViewById(R.id.publish);
		ImageView image = (ImageView) findViewById(R.id.imageView1);
		
		if(isbn.equals("482220296")){
			titleView.setText(appBook[0]);
			autherView.setText(appBook[1]);
			valueView.setText(appBook[2]);
			publishView.setText(appBook[3]);
			image.setImageResource(R.drawable.sample_image);
			
		}else{
			titleView.setText(noData);
			autherView.setText(noData);
			valueView.setText(noData);
			publishView.setText(noData);
			image.setImageResource(R.drawable.ic_launcher);	
		}
	}


	/**
	 * 戻るボタンが押された場合の処理
	 */
	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		if (keyCode == KeyEvent.KEYCODE_BACK) {
			//シリアル通信の終了
			closeSpComm();
		}
		return super.onKeyDown(keyCode, event);
	}
	
	/**
	 * シリアル通信の終了処理
	 */

	public void closeSpComm() {
		if (sp != null) {
			//ネイティブ側でリアルポートの閉鎖を実行
			sp.closeComm();
			sp = null;
		}
	}



	@Override
	public void onPause() {
		super.onPause();
		//シリアル通信の終了
		closeSpComm();
	}
	
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.activity_main, menu);
        return true;
    }
}