package my.pfsample.barcode;

import android.os.Handler;
import android.os.Message;

public class RxtxSerial {

	//ネイティブモジュール（.so）のロード
	static {
		System.loadLibrary("serial-barcode");
	}
	//シリアルデバイスに接続
	public native int openComm(String dev, boolean debug);
	//シリアルデバイスから切断
	public native void closeComm();
	//シリアルデバイスに接続時のパラメータ設定
	private native boolean setCommParams(int baudRate, int dataBits,int stopBits, int parity);
	//ネイティブに自インスタンスを渡す
	protected static native void setListener(RxtxSerial rxtxSerial);

	
	private final static String TAG = "RxtxSerial";
	private OnSerialListener mSerialListener;
	static private Handler mHandler;
	
	// 通信パラメータの設定を行う
	public boolean setCommMode(int baudRate, int dataBits, int stopBits,
			int parity) {
		boolean status = setCommParams(baudRate, dataBits, stopBits, parity);
		return status;
	}

	// イベントリスナーの登録関数
	public void setOnSerialListener(OnSerialListener serialListener) {
		this.mSerialListener = serialListener;
		// ネイティブ側にこのクラスのインスタンスを渡して読み取り開始
		setListener(this);
		mHandler = new Handler() {
			@Override
			public void handleMessage(Message msg) {
				//ハンドラでメッセージを受け取ったらこのリスナーの登録先に渡す
				mSerialListener.onReceive((String) msg.obj); 
			}
		};
	}

	// ネイティブ側からコールされるコールバック関数
	protected static void callBackFromJNI(String code) {
		Message msg = new Message();
		msg.obj = code;
		 // 読み取りコードの値をメッセージに入れてハンドラに渡す
		mHandler.sendMessage(msg);
	}
	
}
